#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const http = require("http");
const http2 = require("http2");
const tls = require("tls");
const log4js = require("log4js");
const yargs = require("yargs");
const piping = require("./piping");
const version_1 = require("./version");
const parser = yargs
    .option("host", {
    describe: "Bind address (e.g. 127.0.0.1, ::1)",
    type: "string",
})
    .option("http-port", {
    describe: "Port of HTTP server",
    default: 8080
})
    .option("enable-https", {
    describe: "Enable HTTPS",
    boolean: true,
    default: false,
})
    .option("https-port", {
    describe: "Port of HTTPS server",
    type: "number"
})
    .option("key-path", {
    describe: "Private key path",
    type: "string"
})
    .option("crt-path", {
    describe: "Certification path",
    type: "string"
});
const args = parser.parseSync(process.argv.slice(2));
const host = args["host"];
const httpPort = args["http-port"];
const enableHttps = args["enable-https"];
const httpsPort = args["https-port"];
const serverKeyPath = args["key-path"];
const serverCrtPath = args["crt-path"];
const logger = log4js.getLogger();
logger.level = "info";
const pipingServer = new piping.Server({ logger });
logger.info(`Piping Server ${version_1.VERSION}`);
http.createServer(pipingServer.generateHandler(false))
    .listen({ host, port: httpPort }, () => {
    logger.info(`Listen HTTP on ${httpPort}...`);
});
if (enableHttps) {
    if (httpsPort === undefined) {
        logger.error("--https-port is required");
        process.exit(1);
    }
    if (serverKeyPath === undefined) {
        logger.error("--key-path is required");
        process.exit(1);
    }
    if (serverCrtPath === undefined) {
        logger.error("--crt-path is required");
        process.exit(1);
    }
    let secureContext;
    const updateSecureContext = () => {
        try {
            secureContext = tls.createSecureContext({
                key: fs.readFileSync(serverKeyPath),
                cert: fs.readFileSync(serverCrtPath),
            });
            logger.info("Certificate loaded");
        }
        catch (e) {
            logger.error("Failed to load certificate", e);
        }
    };
    updateSecureContext();
    if (secureContext === undefined) {
        throw new Error("No certificate");
    }
    fs.watchFile(serverCrtPath, updateSecureContext);
    fs.watchFile(serverKeyPath, updateSecureContext);
    http2.createSecureServer({
        SNICallback: (servername, cb) => cb(null, secureContext),
        allowHTTP1: true
    }, pipingServer.generateHandler(true)).listen({ host, port: httpsPort }, () => {
        logger.info(`Listen HTTPS on ${httpsPort}...`);
    });
}
process.on("uncaughtException", (err) => {
    logger.error("on uncaughtException", err);
});
//# sourceMappingURL=index.js.map